/*
 * Copyright (C) 2011 Ovea <dev@ovea.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
if (window.Store == undefined) {
    (function($) {

        var order = [];
        var stores = {};

        /* Storage API */

        window.Store = function(opts) {
            this.options = $.extend(opts, {});
            if (this.options.order) {
                this.order = this.options.order;
            } else {
                this.order = order;
            }
            this.stores = stores;
            for (var i in order) {
                if (stores[order[i]].check()) {
                    this.impl = stores[order[i]];
                    break;
                }
            }
            if (!this.impl) {
                throw new Error('No appropriate storage found amongst: ' + order);
            }
        };

        window.Store.prototype = {

            set: function(key, obj) {
                if (obj == undefined || obj == null) {
                    return this.del(key)
                } else {
                    var old = this.get(key);
                    this.impl.set(key, obj);
                    return old;
                }
            },

            get: function(key) {
                return this.impl.get(key);
            },

            del: function(key) {
                var old = this.get(key);
                this.impl.del(key);
                return old;
            }

        };

        /* HTML5 Storage */

        order.push('html5');
        stores.html5 = {

            name: 'html5',

            check: function() {
                return 'localStorage' in window;
            },

            set: function(key, obj) {
                localStorage.setItem(key, $.toJSON(obj));
            },

            del: function(key) {
                localStorage.removeItem(key);
            },

            get: function(key) {
                var v = localStorage.getItem(key);
                return v == null ? null : $.parseJSON(v);
            }

        };

        /* Cookie Storage */

        order.push('cookie');
        stores.cookie = {

            name: 'cookie',

            check: function() {
                return "cookie" in document;
            },

            set: function(key, obj, expires) {
                var e = expires == undefined ? 365 : expires;
                var dt = new Date();
                dt.setDate(dt.getDate() + e);
                document.cookie = "CS_" + encodeURIComponent(key) + "=" + encodeURIComponent($.toJSON(obj)) + ";expires=" + dt.toGMTString() + ";path=/";
            },

            del: function(key) {
                this.set(key, '', -1);
            },

            get: function(key) {
                var result = new RegExp('(?:^|; )' + "CS_" + encodeURIComponent(key) + '=([^;]*)').exec(document.cookie);
                return result ? $.parseJSON(decodeURIComponent(result[1])) : null;
            }

        };

        /* In-Page Storage */

        order.push('page');
        stores.page = (function() {

            var data = {};

            return {
                name: 'page',

                check: function() {
                    return true;
                },

                set: function(key, obj) {
                    data[key] = obj;
                },

                del: function(key) {
                    delete data[key];
                },

                get: function(key) {
                    var v = data[key];
                    return v == undefined ? null : v;
                }
            };
        })();


    })(jQuery);
}
